//
//  JDViewController.m
//  JD_Speech_Test
//
//  Created by 满达 on 2018/6/9.
//  Copyright © 2018年 满达. All rights reserved.
//

#import "JDViewController.h"
#import "CommonUtil.h"
#import <JD_Speech/JDAISSpeechManager.h>
#import <JD_Speech/JDAISASRDefines.h>
#import <JD_Speech/JDAISASRParameters.h>

#import "Waver.h"

// Default key
static NSString *const JDAIS_ASR_URL                = @"SpeechContent.URL";
static NSString *const JDAIS_ASR_SOURCE_TYPE        = @"SpeechContent.SOURCE_TYPE";

// Default value
static const NSInteger kSampleRateIndex     = 1;   // 16000
static const NSInteger kSampleRateDef       = EVoiceRecognizeSampleRate16K;
static NSString *const kDomainDef           = @"";
static const NSInteger kRecognizeTypeIndex  = 0;   // Partial
static const NSInteger kRecognizeTypeDef    = 0;
static BOOL            kVadEnabled          = NO;
static BOOL            kVadEnabledDef       = EVoiceRecognizeVADEnabledNO;
static BOOL            kNeedRecord          = NO;
static BOOL            kNeedRecordDef       = EVoiceRecognizeRecordAudioNO;
static NSString *const kAppIDDef            = @"";
static NSString *const kAppKeyDef           = @"";
static NSString *const kSecretKeyDef        = @"";
static NSString *const kServerURLDef        = @"";
static BOOL            kPostProcessEnabled  = NO;
static BOOL            kPostProcessDef      = NO;
static const NSInteger kSourceTypeIndex     = 0;   // 麦克风:0 文件:1
static NSString *const kAudioSourceDef      = @"";
static NSString *const kRecordFilePathDef   = @"";
static BOOL            kLocalVadEnabled     = YES;
static BOOL            kLocalVadEnabledDef  = EVoiceRecognizeLocalVADEnabledYES;
static NSArray  *const kPhraseListDef       = @[];
static const NSInteger kContextIdDef        = -1;
static BOOL            kPartialResultDef    = EVoiceRecognizePartialResultNO;

//#define __SAVE_TO_FILE
#undef __SAVE_TO_FILE

#define __TEST
//#undef __TEST

#ifdef __TEST
// Partial Result测试jingme_mia
// 其他用search
static NSString *const kDomainTestDef          = @"general";//@"search";

// APPID/APPKEY/SECRETKEY具体值请发送到jdai-speech@jd.com进行申请
static NSString *const kApplicationIdTestDef   = @"81c4ca17-9c6b-4704-a4b4-1255bf054868";
static NSString *const kApplicationKeyTestDef  = @"894866b6ce511b438a974bdaa98b619f";
static NSString *const kSecretKeyTestDef       = @"a6eba1a3cedf62cf0347d6440624718a";
// 动态语言模型测试 http://testasr.jd.com/asr
// 京东云网关测试  https://aiapi.jd.com/jdai/asr
// 不经过网关      http://116.196.88.193:80/asr
// Partial Result测试 http://testasr.jd.com/asr
static NSString *const kServerURLTestDef       = @"http://asrapi-base.jd.com/asr";//@"http://116.196.88.193:80/asr";// // 京东云测试url

static NSString *const kRecordFileNameTestDef  = @"asrAudio.pcm";

static NSArray  *const kPhraseListTestDef       = @[@"今天天气怎么样",@"明天天气怎么样",@"今天天气还行"];
static const NSInteger kContextIdTestDef        = 1024;
#endif

@interface JDViewController () <JDAISClientASRDelegate>
{
    BOOL                  _isRunning;
}

@property (weak, nonatomic) IBOutlet UITextView *resultTextView;
@property (weak, nonatomic) IBOutlet UITextView *logTextView;
@property (weak, nonatomic) IBOutlet UIButton *startBtn;
@property (weak, nonatomic) IBOutlet UIButton *stopBtn;
@property (weak, nonatomic) IBOutlet UIButton *cancelBtn;

#ifdef __SAVE_TO_FILE
@property(atomic,strong)  NSString      *outfile;
#endif

@property (weak, nonatomic) IBOutlet UISegmentedControl *sampleRateChoice;
@property (weak, nonatomic) IBOutlet UITextField *domainText;
@property (weak, nonatomic) IBOutlet UISegmentedControl *recognizeTypeChoice;
@property (weak, nonatomic) IBOutlet UISwitch *toggleVAD;
@property (weak, nonatomic) IBOutlet UISwitch *toggleRecordAudio;
@property (weak, nonatomic) IBOutlet UITextField *APPID;
@property (weak, nonatomic) IBOutlet UITextField *serverURL;
@property (weak, nonatomic) IBOutlet UISwitch *togglePostProcess;
@property (weak, nonatomic) IBOutlet UISegmentedControl *audioSourceChoice;
@property (weak, nonatomic) IBOutlet UITextField *audioSource;
@property (weak, nonatomic) IBOutlet UITextField *recordFileName;
@property (weak, nonatomic) IBOutlet UITextField *vadStartDelayInput;
@property (weak, nonatomic) IBOutlet UITextField *vadEndDelayInput;
@property (weak, nonatomic) IBOutlet UISwitch *toggleLocalVAD;
@property (weak, nonatomic) IBOutlet UITextField *contextId;
@property (weak, nonatomic) IBOutlet UITextField *contextList;
@property (weak, nonatomic) IBOutlet UISwitch *togglePartialResult;

- (IBAction)sampleRateDidChoice:(id)sender;
- (IBAction)domainTextEdit:(id)sender;
- (IBAction)recognizeTypeDidChoice:(id)sender;
- (IBAction)toggleVAD:(id)sender;
- (IBAction)toggleRecordFile:(id)sender;
- (IBAction)APPIDEdit:(id)sender;
- (IBAction)serverURLEdit:(id)sender;
- (IBAction)togglePostProcess:(id)sender;
- (IBAction)audioSourceChoice:(id)sender;
- (IBAction)audioSourceEdit:(id)sender;
- (IBAction)recordFileNameEdit:(id)sender;
- (IBAction)vadStartDelayEdit:(id)sender;
- (IBAction)vadEndDelayEdit:(id)sender;
- (IBAction)toggleLocalVAD:(id)sender;
- (IBAction)contextIdEdit:(id)sender;
- (IBAction)contextListEdit:(id)sender;
- (IBAction)togglePartialResult:(id)sender;

- (IBAction)defaultSet:(id)sender;

//-----------------------------------------------------------------------------------
@property(nonatomic,strong) JDAISSpeechManager   *JDSpeechManager;             //语音管理库
@property(nonatomic,strong) NSDictionary         *asrSpeechManagerConfig;      //语音配置参数
@property(nonatomic,assign)   float               averagePower;                // 语音分贝大小

- (void)mySetDefault;
- (void)myViewLoad;
@end

@implementation JDViewController

#pragma mark --defaultMethods
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    _isRunning = NO;
    [self cleanLogUI];
    [self onInitializing];
    
    [self myViewLoad];
    
    self.logTextView.backgroundColor = [UIColor grayColor];
    
    Waver * waver = [[Waver alloc] initWithFrame:CGRectMake(0, CGRectGetHeight(self.view.bounds)/2.0 + 100.0, CGRectGetWidth(self.view.bounds), 100.0)];
    waver.waverLevelCallback = ^(Waver * waver) {
//        self.averagePower = 0.0;
//        CGFloat normalizedValue = pow (10, self.averagePower / 40);
        CGFloat normalizedValue = self.averagePower / 40;
        waver.level = normalizedValue;
    };
    [self.view addSubview:waver];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)viewWillDisappear:(BOOL)animated {
    if (_isRunning) {
        [self onCancelWorking0];
        [self.JDSpeechManager send:JDAIS_ASR_CMD_CANCEL withParams:nil];
        [self onCancelWorking1];
        _isRunning = NO;
    }
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

#pragma mark - UI Button Helper

- (void)cleanLogUI
{
//    self.resultTextView.text = @"";
    self.logTextView.text    = @"";
}

- (void)cleanResultLogUI
{
    self.resultTextView.text = @"";
}

- (void)onInitializing
{
    [self.startBtn setTitle:@"开始识别" forState:UIControlStateNormal];
    self.startBtn.enabled   = YES;
    self.stopBtn.enabled    = NO;
    self.cancelBtn.enabled  = NO;
}
- (void)onStartWorking0
{
    //[self.startBtn setTitle:@"识别中..." forState:UIControlStateNormal];
    self.startBtn.enabled   = NO;
    self.stopBtn.enabled    = NO;
    self.cancelBtn.enabled  = NO;
}
- (void)onStartWorking1
{
    [self.startBtn setTitle:@"识别中..." forState:UIControlStateNormal];
    self.startBtn.enabled   = NO;
    self.stopBtn.enabled    = YES;
    self.cancelBtn.enabled  = YES;
}
-(void)onStopWorking
{
    //[self.startBtn setTitle:@"开始识别" forState:UIControlStateNormal];
    self.startBtn.enabled   = NO;
    self.stopBtn.enabled    = NO;
    self.cancelBtn.enabled  = NO;
}
-(void)onCancelWorking0
{
    self.startBtn.enabled   = NO;
    self.stopBtn.enabled    = NO;
    self.cancelBtn.enabled  = NO;
}
-(void)onCancelWorking1
{
    [self onInitializing];
}

- (IBAction)audioStart:(id)sender {
    if (_isRunning) {
        NSLog(@"haven't finished");
//        [self printLogTextView:@"haven't finished"];
        return;
    }
    
    [self cleanLogUI];
    NSLog(@"开始识别");
    [self printLogTextView:@"开始识别"];
    _isRunning = YES;
    [self onStartWorking0];
#ifdef __SAVE_TO_FILE
    // 创建一个file
    [self creatOutFile];
#endif
    [self constructParams];
    [self.JDSpeechManager send:JDAIS_ASR_CMD_START withParams:[JDViewController convertToJsonData:_asrSpeechManagerConfig]];
}

- (IBAction)audioStop:(id)sender {
    if (_isRunning) {
        NSLog(@"停止识别");
        [self printLogTextView:@"停止识别"];
        [self onStopWorking];
        
        [self.JDSpeechManager send:JDAIS_ASR_CMD_STOP withParams:nil];
    }
}

- (IBAction)audioCancel:(id)sender {
    if (_isRunning) {
        NSLog(@"取消识别");
        [self printLogTextView:@"取消识别"];
        
        [self onCancelWorking0];
        [self.JDSpeechManager send:JDAIS_ASR_CMD_CANCEL withParams:nil];
        [self onCancelWorking1];
        _isRunning = NO;
    }
}

- (void)printLogTextView:(NSString *)logString
{
    self.logTextView.text = [[NSString stringWithFormat:@"%@\r\n",logString] stringByAppendingString:self.logTextView.text];
    [self.logTextView scrollRangeToVisible:NSMakeRange(0, 0)];
}

- (void)printResTextView:(NSString *)logString
{
    self.resultTextView.text = [[NSString stringWithFormat:@"%@\r\n",logString] stringByAppendingString:self.resultTextView.text];
    [self.resultTextView scrollRangeToVisible:NSMakeRange(0, 0)];
}

#pragma mark --JDAISClientASRDelegate

- (void) onEventSpeechRecognition : (NSString *)event withResult : (NSString *)result withData : (NSData *)data {
    
    __weak typeof(self) weakSelf = self;
    dispatch_async(dispatch_get_main_queue(), ^{
        __strong JDViewController *strongSelf = weakSelf;
        if (strongSelf) {
            NSDictionary *resultDic = [JDViewController dictionaryWithJsonString:result];
            
            NSString *errLog = [NSString stringWithFormat:@"Event %@", event];
            NSLog(@"%@", errLog);
//            [strongSelf printLogTextView:errLog];
            
            if ([event isEqualToString: EV_SJDAIS_ASR_SpeechStart]){
                // 准备就绪
                [strongSelf onStartWorking1];
                
            } else if ([event isEqualToString: EV_SJDAIS_ASR_SpeechBegin]) {
                // Speech Begin
                
            } else if ([event isEqualToString: EV_SJDAIS_ASR_SpeechEnd]) {
                // Speech End
                strongSelf->_averagePower = 0.0;
            } else if ([event isEqualToString: EV_SJDAIS_ASR_SpeechVolume]) {
                strongSelf->_averagePower = [resultDic[@"volume"] floatValue];
                NSString *errLog = [NSString stringWithFormat:@"Volume:%f", strongSelf->_averagePower];
                NSLog(@"%@", errLog);
            } else if ([event isEqualToString: EV_SJDAIS_ASR_SpeechParitalResult]) {
                // 部分结果，一句话识别不会出现
                NSInteger err_code    = [resultDic[@"err_code"] intValue];
                NSString *requestId   = resultDic[@"request_id"];
                NSString *err_msg     = resultDic[@"err_msg"];
                NSDictionary *context = resultDic[@"context"];
                
                if (err_code == EVRClientErrorCodeOK) {
                    if (context) {
                        NSString *contextText = [strongSelf getDescriptionForDic:context];
                        if (context[@"text"]) {
                            [self cleanResultLogUI];
                        }
                        [strongSelf printResTextView:context[@"text"]];
//                        [strongSelf printLogTextView:[NSString stringWithFormat:@"PartialResult:%@", contextText]];
                    }
#ifdef __SAVE_TO_FILE
                    // Write to file
                    if (data && data.length > 0) {
                        NSLog(@"Data length:%ld", data.length);
                        [strongSelf outToFile:data];
                    }
#endif
                }
                
                NSLog(@"err_code:%ld,request_id:%@,error:%@", err_code, requestId, err_msg);
                NSString *errStr = [NSString stringWithFormat:@"err_code:%ld,request_id:%@,error:%@",err_code, requestId, err_msg];
//                [strongSelf printLogTextView:errStr];
                
            } else if ([event isEqualToString: EV_SJDAIS_ASR_SpeechResult]) {
                // 最终结果
                NSInteger err_code    = [resultDic[@"err_code"] intValue];
                NSString *requestId   = resultDic[@"request_id"];
                NSString *err_msg     = resultDic[@"err_msg"];
                NSDictionary *context = resultDic[@"context"];
                
                [self cleanResultLogUI];
                
                if (err_code == EVRClientErrorCodeOK) {
                    if (context)
                    {
                        NSString *contextText = [self getDescriptionForDic:context];
                        [strongSelf printResTextView:context[@"text"]];
//                        [strongSelf printLogTextView:[NSString stringWithFormat:@"FinalResult:%@", contextText]];
                    }
#ifdef __SAVE_TO_FILE
                    // Write to file
                    if (data && data.length > 0) {
                        NSLog(@"Data length:%ld", data.length);
                        [strongSelf outToFile:data];
                    }
#endif
                }
                
                NSLog(@"err_code:%ld,request_id:%@,error:%@", err_code, requestId, err_msg);
                NSString *errStr = [NSString stringWithFormat:@"err_code:%ld,request_id:%@,error:%@",err_code, requestId, err_msg];
                [strongSelf printLogTextView:errStr];
                
                [strongSelf onInitializing];
                strongSelf->_isRunning = NO;
                
            } else {
                NSLog(@"Unknown event");
//                [strongSelf printLogTextView:@"Unknown event"];
            }
        }
    });
}

- (NSString *)getDescriptionForDic:(NSDictionary *)dic {
    if (dic) {
        return [[NSString alloc] initWithData:[NSJSONSerialization dataWithJSONObject:dic
                                                                              options:NSJSONWritingPrettyPrinted
                                                                                error:nil] encoding:NSUTF8StringEncoding];
    }
    return nil;
}

#pragma mark --constructParams

- (void)constructParams{

    NSInteger sampleRateVal;
    NSString *domainVal;
    NSInteger recognizeTypeVal;
    NSInteger vadEnabledVal;
    NSInteger recordAudioVal;
    NSString *appIDVal;
    NSString *appKeyVal;
    NSString *secretKeyVal;
    NSString *serverURLVal;
    NSInteger postProcessVal;
    NSInteger partialResultVal;
    NSInteger audioSourceType;
    NSString *audioSourceVal;
    NSString *recordFileName;
    float vadStartDelay;
    float vadEndDelay;
    NSInteger localVadEnabledVal;
    
    NSInteger puncProcess;
//    NSInteger dynamicLm;

    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    
    if([userDefaults objectForKey:JDAIS_ASR_SAMPLE]) {
        NSInteger index = [userDefaults integerForKey:JDAIS_ASR_SAMPLE];
        if (index == 0)    // 8k
            sampleRateVal = EVoiceRecognizeSampleRate8K;
        else if (index == 1) // 16k
            sampleRateVal = EVoiceRecognizeSampleRate16K;
    } else {
        sampleRateVal = kSampleRateDef;
    }
    
    if([userDefaults objectForKey:JDAIS_ASR_DOMAIN]) {
        domainVal = [userDefaults stringForKey:JDAIS_ASR_DOMAIN];
    } else {
        domainVal = kDomainDef;
    }
    
    if([userDefaults objectForKey:JDAIS_ASR_RECOGNIZE_TYPE]) {
        recognizeTypeVal = [userDefaults integerForKey:JDAIS_ASR_RECOGNIZE_TYPE];
    } else {
        recognizeTypeVal = kRecognizeTypeDef;
    }
    
    if([userDefaults objectForKey:JDAIS_ASR_SERVER_VAD_ENABLE]) {
        vadEnabledVal = [userDefaults boolForKey:JDAIS_ASR_SERVER_VAD_ENABLE] ? EVoiceRecognizeVADEnabledYES : EVoiceRecognizeVADEnabledNO;
    } else {
        vadEnabledVal = kVadEnabledDef;
    }
    
    if([userDefaults objectForKey:JDAIS_ASR_IS_NEED_RECORD_FILE]) {
        recordAudioVal = [userDefaults boolForKey:JDAIS_ASR_IS_NEED_RECORD_FILE] ? EVoiceRecognizeRecordAudioYes: EVoiceRecognizeRecordAudioNO;
    } else {
        recordAudioVal = kNeedRecordDef;
    }

    if([userDefaults stringForKey:JDAIS_ASR_APPID]) {
        appIDVal = [userDefaults stringForKey:JDAIS_ASR_APPID];
    } else {
        appIDVal = kAppIDDef;
    }
    
    if([userDefaults stringForKey:JDAIS_ASR_APPKEY]) {
        appKeyVal = [userDefaults stringForKey:JDAIS_ASR_APPKEY];
    } else {
        appKeyVal = kAppKeyDef;
    }
    
    if([userDefaults stringForKey:JDAIS_ASR_SECRETKEY]) {
        secretKeyVal = [userDefaults stringForKey:JDAIS_ASR_SECRETKEY];
    } else {
        secretKeyVal = kSecretKeyDef;
    }
    
    if([userDefaults stringForKey:JDAIS_ASR_URL]) {
        serverURLVal = [userDefaults stringForKey:JDAIS_ASR_URL];
    } else {
        serverURLVal = kServerURLDef;
    }
    
    if([userDefaults boolForKey:JDAIS_ASR_POST_PROCESS_ENABLE]) {
        postProcessVal = [userDefaults boolForKey:JDAIS_ASR_POST_PROCESS_ENABLE] ? EVoiceRecognizePostProcessYes : EVoiceRecognizePostProcessNO;
    } else {
        postProcessVal = kPostProcessDef;
    }
    
    if([userDefaults objectForKey:JDAIS_ASR_SOURCE_TYPE]) {
        audioSourceType = [userDefaults integerForKey:JDAIS_ASR_SOURCE_TYPE];
    } else {
        audioSourceType = kSourceTypeIndex;
    }
    
    if([userDefaults stringForKey:JDAIS_ASR_AUDIO_SOURCE]) {
        audioSourceVal = [userDefaults stringForKey:JDAIS_ASR_AUDIO_SOURCE];
    } else {
        audioSourceVal = kAudioSourceDef;
    }

    if([userDefaults stringForKey:JDAIS_ASR_RECORD_FILE_PATH]) {
        recordFileName = [userDefaults stringForKey:JDAIS_ASR_RECORD_FILE_PATH];
    } else {
        recordFileName = kRecordFilePathDef;
    }
    
    if([userDefaults stringForKey:JDAIS_ASR_VAD_START_DELAY]) {
        vadStartDelay = [[userDefaults stringForKey:JDAIS_ASR_VAD_START_DELAY] floatValue];
    } else {
        vadStartDelay = kJDAIS_ASR_VAD_START_DELAY_DEF_VAL;
    }
    
    if([userDefaults stringForKey:JDAIS_ASR_VAD_END_DELAY]) {
        vadEndDelay = [[userDefaults stringForKey:JDAIS_ASR_VAD_END_DELAY] floatValue];
    } else {
        vadEndDelay = kJDAIS_ASR_VAD_END_DELAY_DEF_VAL;
    }
    
    if([userDefaults objectForKey:JDAIS_ASR_LOCAL_VAD_ENABLE]) {
        localVadEnabledVal = [userDefaults boolForKey:JDAIS_ASR_LOCAL_VAD_ENABLE] ? EVoiceRecognizeLocalVADEnabledYES : EVoiceRecognizeLocalVADEnabledNO;
    } else {
        localVadEnabledVal = kLocalVadEnabledDef;
    }
    
    NSString *phraseList;
    NSInteger contextId;
    
    if([userDefaults stringForKey:JDAIS_ASR_CONTEXT_ID]) {
        contextId = [[userDefaults stringForKey:JDAIS_ASR_CONTEXT_ID] integerValue];
    } else {
        contextId = kContextIdDef;
    }
    
    if([userDefaults stringForKey:JDAIS_ASR_PHRASE_LIST]) {
        phraseList = [userDefaults stringForKey:JDAIS_ASR_PHRASE_LIST];
    } else {
        phraseList = [CommonUtil objectToJson:kPhraseListDef];
    }
    
    if([userDefaults boolForKey:JDAIS_ASR_PARTIAL_RESULT]) {
        partialResultVal = [userDefaults boolForKey:JDAIS_ASR_PARTIAL_RESULT] ? EVoiceRecognizePartialResultYes : EVoiceRecognizePartialResultNO;
    } else {
        partialResultVal = kPartialResultDef;
    }
    
    puncProcess = 1;
//    dynamicLm = 1;
    
#ifdef __TEST
    if ([domainVal isEqualToString:@""]) {
        domainVal = kDomainTestDef;
    }
    if ([appIDVal isEqualToString:@""]) {
        appIDVal = kApplicationIdTestDef;
    }
    if ([appKeyVal isEqualToString:@""]) {
        appKeyVal = kApplicationKeyTestDef;
    }
    if ([secretKeyVal isEqualToString:@""]) {
        secretKeyVal = kSecretKeyTestDef;
    }
    if ([serverURLVal isEqualToString:@""]) {
        serverURLVal = kServerURLTestDef;
    }
    if ([recordFileName isEqualToString:@""]) {
        recordFileName = kRecordFileNameTestDef;
    }
    if (contextId == -1) {
        contextId = kContextIdTestDef;
    }
    if ([phraseList isEqualToString:@"[]"] || [phraseList isEqualToString:@""]) {
        phraseList = [CommonUtil objectToJson:kPhraseListTestDef];
    }
#endif
    
    if (audioSourceType == 0) { // 麦克风
        NSString *path = [[NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0] stringByAppendingPathComponent:recordFileName]; // 录音后的格式为pcm
        if ([serverURLVal isEqualToString:@""]) {
            self.asrSpeechManagerConfig = @{
                                        JDAIS_ASR_SAMPLE:@(sampleRateVal),
                                        JDAIS_ASR_DOMAIN:domainVal,
                                        JDAIS_ASR_RECOGNIZE_TYPE:@(recognizeTypeVal),
                                        JDAIS_ASR_SERVER_VAD_ENABLE:@(vadEnabledVal),
                                        JDAIS_ASR_IS_NEED_RECORD_FILE:@(recordAudioVal),
                                        JDAIS_ASR_APPID:appIDVal,
                                        JDAIS_ASR_APPKEY:appKeyVal,
                                        JDAIS_ASR_SECRETKEY:secretKeyVal,
                                        JDAIS_ASR_POST_PROCESS_ENABLE:@(postProcessVal),
                                        JDAIS_ASR_RECORD_FILE_PATH:path,
                                        JDAIS_ASR_VAD_START_DELAY:@(vadStartDelay),
                                        JDAIS_ASR_VAD_END_DELAY:@(vadEndDelay),
                                        JDAIS_ASR_LOCAL_VAD_ENABLE:@(localVadEnabledVal),
                                        JDAIS_ASR_PHRASE_LIST:phraseList,
                                        JDAIS_ASR_CONTEXT_ID:@(contextId),
                                        JDAIS_ASR_PARTIAL_RESULT:@(partialResultVal),
                                        JDAIS_ASR_PUNC_PROCESS:@(puncProcess),
//                                        JDAIS_ASR_DYNAMIC_LM_ENABLE:@(dynamicLm),
                                        };
        } else {
            self.asrSpeechManagerConfig = @{
                                            JDAIS_ASR_SAMPLE:@(sampleRateVal),
                                            JDAIS_ASR_DOMAIN:domainVal,
                                            JDAIS_ASR_RECOGNIZE_TYPE:@(recognizeTypeVal),
                                            JDAIS_ASR_SERVER_VAD_ENABLE:@(vadEnabledVal),
                                            JDAIS_ASR_IS_NEED_RECORD_FILE:@(recordAudioVal),
                                            JDAIS_ASR_APPID:appIDVal,
                                            JDAIS_ASR_APPKEY:appKeyVal,
                                            JDAIS_ASR_SECRETKEY:secretKeyVal,
                                            JDAIS_ASR_URL:serverURLVal,
                                            JDAIS_ASR_POST_PROCESS_ENABLE:@(postProcessVal),
                                            JDAIS_ASR_RECORD_FILE_PATH:path,
                                            JDAIS_ASR_VAD_START_DELAY:@(vadStartDelay),
                                            JDAIS_ASR_VAD_END_DELAY:@(vadEndDelay),
                                            JDAIS_ASR_LOCAL_VAD_ENABLE:@(localVadEnabledVal),
                                            JDAIS_ASR_PHRASE_LIST:phraseList,
                                            JDAIS_ASR_CONTEXT_ID:@(contextId),
                                            JDAIS_ASR_PARTIAL_RESULT:@(partialResultVal),
                                            JDAIS_ASR_PUNC_PROCESS:@(puncProcess),
//                                            JDAIS_ASR_DYNAMIC_LM_ENABLE:@(dynamicLm),
                                            };
        }
        
    } else if (audioSourceType == 1) { // 文件
        NSString *audioSource = [[NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0] stringByAppendingPathComponent:audioSourceVal]; // Documents下的文件
#ifdef __TEST
        if ([audioSourceVal isEqualToString:@""]) {
            audioSource = [[[NSBundle mainBundle] bundlePath] stringByAppendingPathComponent:@"HelloJD.pcm"];
        }
#endif
        if ([serverURLVal isEqualToString:@""]) {
            self.asrSpeechManagerConfig = @{
                                        JDAIS_ASR_SAMPLE:@(sampleRateVal),
                                        JDAIS_ASR_DOMAIN:domainVal,
                                        JDAIS_ASR_RECOGNIZE_TYPE:@(recognizeTypeVal),
                                        JDAIS_ASR_SERVER_VAD_ENABLE:@(vadEnabledVal),
                                        JDAIS_ASR_APPID:appIDVal,
                                        JDAIS_ASR_APPKEY:appKeyVal,
                                        JDAIS_ASR_SECRETKEY:secretKeyVal,
                                        JDAIS_ASR_POST_PROCESS_ENABLE:@(postProcessVal),
                                        JDAIS_ASR_AUDIO_SOURCE:audioSource,
                                        JDAIS_ASR_PHRASE_LIST:phraseList,
                                        JDAIS_ASR_CONTEXT_ID:@(contextId),
                                        JDAIS_ASR_PARTIAL_RESULT:@(partialResultVal),
                                        JDAIS_ASR_PUNC_PROCESS:@(puncProcess),
//                                        JDAIS_ASR_DYNAMIC_LM_ENABLE:@(dynamicLm),
                                        };
        } else {
            self.asrSpeechManagerConfig = @{
                                            JDAIS_ASR_SAMPLE:@(sampleRateVal),
                                            JDAIS_ASR_DOMAIN:domainVal,
                                            JDAIS_ASR_RECOGNIZE_TYPE:@(recognizeTypeVal),
                                            JDAIS_ASR_SERVER_VAD_ENABLE:@(vadEnabledVal),
                                            JDAIS_ASR_APPID:appIDVal,
                                            JDAIS_ASR_APPKEY:appKeyVal,
                                            JDAIS_ASR_SECRETKEY:secretKeyVal,
                                            JDAIS_ASR_URL:serverURLVal,
                                            JDAIS_ASR_POST_PROCESS_ENABLE:@(postProcessVal),
                                            JDAIS_ASR_AUDIO_SOURCE:audioSource,
                                            JDAIS_ASR_PHRASE_LIST:phraseList,
                                            JDAIS_ASR_CONTEXT_ID:@(contextId),
                                            JDAIS_ASR_PARTIAL_RESULT:@(partialResultVal),
                                            JDAIS_ASR_PUNC_PROCESS:@(puncProcess),
//                                            JDAIS_ASR_DYNAMIC_LM_ENABLE:@(dynamicLm),
                                            };
        }
    } else {
        NSLog(@"Unknown audio source type");
    }
}

#pragma mark --JDSpeechManager
- (JDAISSpeechManager *)JDSpeechManager{
    if(!_JDSpeechManager)
    {
        _JDSpeechManager = [JDAISSpeechManager create:JDAIS_ENGINETYPE_ASR];
        [self printLogTextView:[NSString stringWithFormat:@"SDK版本：%@",[_JDSpeechManager getSDKVersion]]];
        [_JDSpeechManager setListener:self];
    }
    return _JDSpeechManager;
}

#pragma mark --Settings
// 更改配置并且存储到默认设置

- (IBAction)sampleRateDidChoice:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setInteger:self.sampleRateChoice.selectedSegmentIndex forKey:JDAIS_ASR_SAMPLE];
    [userDefaults synchronize];
}

- (IBAction)domainTextEdit:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:self.domainText.text forKey:JDAIS_ASR_DOMAIN];
    [userDefaults synchronize];
}

- (IBAction)recognizeTypeDidChoice:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setInteger:self.recognizeTypeChoice.selectedSegmentIndex forKey:JDAIS_ASR_RECOGNIZE_TYPE];
    [userDefaults synchronize];
}

- (IBAction)toggleVAD:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setBool:self.toggleVAD.on forKey:JDAIS_ASR_SERVER_VAD_ENABLE];
    [userDefaults synchronize];
}

- (IBAction)toggleRecordFile:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setBool:self.toggleRecordAudio.on forKey:JDAIS_ASR_IS_NEED_RECORD_FILE];
    [userDefaults synchronize];
}

- (IBAction)APPIDEdit:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:self.APPID.text forKey:JDAIS_ASR_APPID];
    [userDefaults synchronize];
}

- (IBAction)serverURLEdit:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:self.serverURL.text forKey:JDAIS_ASR_URL];
    [userDefaults synchronize];
}

- (IBAction)togglePostProcess:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setBool:self.togglePostProcess.on forKey:JDAIS_ASR_POST_PROCESS_ENABLE];
    [userDefaults synchronize];
}

- (IBAction)audioSourceChoice:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setInteger:self.audioSourceChoice.selectedSegmentIndex forKey:JDAIS_ASR_SOURCE_TYPE];
    [userDefaults synchronize];
}

- (IBAction)audioSourceEdit:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:self.audioSource.text forKey:JDAIS_ASR_AUDIO_SOURCE];
    [userDefaults synchronize];
}

- (IBAction)recordFileNameEdit:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:self.recordFileName.text forKey:JDAIS_ASR_RECORD_FILE_PATH];
    [userDefaults synchronize];
}

- (IBAction)vadStartDelayEdit:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:self.vadStartDelayInput.text forKey:JDAIS_ASR_VAD_START_DELAY];
    [userDefaults synchronize];
}

- (IBAction)vadEndDelayEdit:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:self.vadEndDelayInput.text forKey:JDAIS_ASR_VAD_END_DELAY];
    [userDefaults synchronize];
}

- (IBAction)toggleLocalVAD:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setBool:self.toggleLocalVAD.on forKey:JDAIS_ASR_LOCAL_VAD_ENABLE];
    [userDefaults synchronize];
}

- (IBAction)contextIdEdit:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:self.contextId.text forKey:JDAIS_ASR_CONTEXT_ID];
    [userDefaults synchronize];
}

- (IBAction)contextListEdit:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:self.contextList.text forKey:JDAIS_ASR_PHRASE_LIST];
    [userDefaults synchronize];
}

- (IBAction)togglePartialResult:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setBool:self.togglePartialResult.on forKey:JDAIS_ASR_PARTIAL_RESULT];
    [userDefaults synchronize];
}

- (IBAction)defaultSet:(id)sender {
    [self mySetDefault];
}

// 恢复到默认设置
- (void)mySetDefault {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];

    self.sampleRateChoice.selectedSegmentIndex = kSampleRateIndex;
    [userDefaults setInteger:self.sampleRateChoice.selectedSegmentIndex forKey:JDAIS_ASR_SAMPLE];
    
    self.domainText.text = kDomainDef;
    [userDefaults setObject:self.domainText.text forKey:JDAIS_ASR_DOMAIN];
    
    self.recognizeTypeChoice.selectedSegmentIndex = kRecognizeTypeIndex;
    [userDefaults setInteger:self.recognizeTypeChoice.selectedSegmentIndex forKey:JDAIS_ASR_RECOGNIZE_TYPE];
    
    self.toggleVAD.on = kVadEnabled;
    [userDefaults setBool:self.toggleVAD.on forKey:JDAIS_ASR_SERVER_VAD_ENABLE];
    
    self.toggleRecordAudio.on = kNeedRecord;
    [userDefaults setBool:self.toggleRecordAudio.on forKey:JDAIS_ASR_IS_NEED_RECORD_FILE];
    
    self.APPID.text = kAppIDDef;
    [userDefaults setObject:self.APPID.text forKey:JDAIS_ASR_APPID];
    
    self.serverURL.text = kServerURLDef;
    [userDefaults setObject:self.serverURL.text forKey:JDAIS_ASR_URL];
    
    self.togglePostProcess.on = kPostProcessEnabled;
    [userDefaults setBool:self.togglePostProcess.on forKey:JDAIS_ASR_POST_PROCESS_ENABLE];
    
    self.audioSource.text = kAudioSourceDef;
    [userDefaults setObject:self.audioSource.text forKey:JDAIS_ASR_AUDIO_SOURCE];
    
    self.recordFileName.text = kRecordFilePathDef;
    [userDefaults setObject:self.recordFileName.text forKey:JDAIS_ASR_RECORD_FILE_PATH];
    
    self.vadStartDelayInput.text = [NSString stringWithFormat:@"%f", kJDAIS_ASR_VAD_START_DELAY_DEF_VAL];
    [userDefaults setObject:self.vadStartDelayInput.text forKey:JDAIS_ASR_VAD_START_DELAY];
    
    self.vadEndDelayInput.text = [NSString stringWithFormat:@"%f", kJDAIS_ASR_VAD_END_DELAY_DEF_VAL];
    [userDefaults setObject:self.vadEndDelayInput.text forKey:JDAIS_ASR_VAD_END_DELAY];
    
    self.toggleLocalVAD.on = kLocalVadEnabled;
    [userDefaults setBool:self.toggleLocalVAD.on forKey:JDAIS_ASR_LOCAL_VAD_ENABLE];
    
    self.contextId.text = [NSString stringWithFormat:@"%ld", kContextIdDef];
    [userDefaults setObject:self.contextId.text forKey:JDAIS_ASR_CONTEXT_ID];
    
    self.contextList.text = [CommonUtil objectToJson:kPhraseListDef];
    [userDefaults setObject:self.contextList.text forKey:JDAIS_ASR_PHRASE_LIST];
    
    self.togglePartialResult.on = kPartialResultDef;
    [userDefaults setBool:self.togglePartialResult.on forKey:JDAIS_ASR_PARTIAL_RESULT];

    [userDefaults synchronize];
}

// 加载默认设置选项
- (void)myViewLoad {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    
    if([userDefaults objectForKey:JDAIS_ASR_SAMPLE]) {
        NSInteger index = [userDefaults integerForKey:JDAIS_ASR_SAMPLE];
        [self.sampleRateChoice setSelectedSegmentIndex:index];
    } else {
        [self.sampleRateChoice setSelectedSegmentIndex:kSampleRateIndex];
    }
    
    if([userDefaults stringForKey:JDAIS_ASR_DOMAIN]) {
        self.domainText.text = [userDefaults stringForKey:JDAIS_ASR_DOMAIN];
    } else {
        self.domainText.text = kDomainDef;
    }
    
    if([userDefaults objectForKey:JDAIS_ASR_RECOGNIZE_TYPE]) {
        NSInteger index = [userDefaults integerForKey:JDAIS_ASR_RECOGNIZE_TYPE];
        [self.recognizeTypeChoice setSelectedSegmentIndex:index];
    } else {
        [self.recognizeTypeChoice setSelectedSegmentIndex:kRecognizeTypeIndex];
    }
    
    if([userDefaults objectForKey:JDAIS_ASR_SERVER_VAD_ENABLE]) {
        self.toggleVAD.on = [userDefaults boolForKey:JDAIS_ASR_SERVER_VAD_ENABLE];
    } else {
        self.toggleVAD.on = kVadEnabled;
    }
    
    if([userDefaults objectForKey:JDAIS_ASR_IS_NEED_RECORD_FILE]) {
        self.toggleRecordAudio.on = [userDefaults boolForKey:JDAIS_ASR_IS_NEED_RECORD_FILE];
    } else {
        self.toggleRecordAudio.on = kNeedRecord;
    }
    
    if([userDefaults objectForKey:JDAIS_ASR_APPID]) {
        self.APPID.text = [userDefaults stringForKey:JDAIS_ASR_APPID];
    } else {
        self.APPID.text = kAppIDDef;
    }
    
    if([userDefaults objectForKey:JDAIS_ASR_URL]) {
        self.serverURL.text = [userDefaults stringForKey:JDAIS_ASR_URL];
    } else {
        self.serverURL.text = kServerURLDef;
    }
    
    if([userDefaults objectForKey:JDAIS_ASR_POST_PROCESS_ENABLE]) {
        self.togglePostProcess.on = [userDefaults boolForKey:JDAIS_ASR_POST_PROCESS_ENABLE];
    } else {
        self.togglePostProcess.on = kPostProcessEnabled;
    }
    
    if([userDefaults objectForKey:JDAIS_ASR_SOURCE_TYPE]) {
        NSInteger index = [userDefaults integerForKey:JDAIS_ASR_SOURCE_TYPE];
        [self.audioSourceChoice setSelectedSegmentIndex:index];
    } else {
        [self.audioSourceChoice setSelectedSegmentIndex:kSourceTypeIndex];
    }
    
    if([userDefaults stringForKey:JDAIS_ASR_AUDIO_SOURCE]) {
        self.audioSource.text = [userDefaults stringForKey:JDAIS_ASR_AUDIO_SOURCE];
    } else {
        self.audioSource.text = kAudioSourceDef;
    }
    
    if([userDefaults stringForKey:JDAIS_ASR_RECORD_FILE_PATH]) {
        self.recordFileName.text = [userDefaults stringForKey:JDAIS_ASR_RECORD_FILE_PATH];
    } else {
        self.recordFileName.text = kRecordFilePathDef;
    }
    
    if([userDefaults stringForKey:JDAIS_ASR_VAD_START_DELAY]) {
        self.vadStartDelayInput.text = [userDefaults stringForKey:JDAIS_ASR_VAD_START_DELAY];
    } else {
        self.vadStartDelayInput.text = [NSString stringWithFormat:@"%f", kJDAIS_ASR_VAD_START_DELAY_DEF_VAL];
    }
    
    if([userDefaults stringForKey:JDAIS_ASR_VAD_END_DELAY]) {
        self.vadEndDelayInput.text = [userDefaults stringForKey:JDAIS_ASR_VAD_END_DELAY];
    } else {
        self.vadEndDelayInput.text = [NSString stringWithFormat:@"%f", kJDAIS_ASR_VAD_END_DELAY_DEF_VAL];
    }
    
    if([userDefaults objectForKey:JDAIS_ASR_LOCAL_VAD_ENABLE]) {
        self.toggleLocalVAD.on = [userDefaults boolForKey:JDAIS_ASR_LOCAL_VAD_ENABLE];
    } else {
        self.toggleLocalVAD.on = kLocalVadEnabled;
    }
    
    if([userDefaults stringForKey:JDAIS_ASR_CONTEXT_ID]) {
        self.contextId.text = [userDefaults stringForKey:JDAIS_ASR_CONTEXT_ID];
    } else {
        self.contextId.text = [NSString stringWithFormat:@"%ld", kContextIdDef];
    }
    
    if([userDefaults stringForKey:JDAIS_ASR_PHRASE_LIST]) {
        self.contextList.text = [userDefaults stringForKey:JDAIS_ASR_PHRASE_LIST];
    } else {
        self.contextList.text = [CommonUtil objectToJson:kPhraseListDef];
    }
    
    if([userDefaults objectForKey:JDAIS_ASR_PARTIAL_RESULT]) {
        self.togglePartialResult.on = [userDefaults boolForKey:JDAIS_ASR_PARTIAL_RESULT];
    } else {
        self.togglePartialResult.on = kPartialResultDef;
    }
}

#ifdef __SAVE_TO_FILE
- (void) creatOutFile
{
    // 获取Documents路径
    NSArray *path = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [path objectAtIndex:0];
    
    // 获取文件路径
    NSTimeInterval timestamp = [NSDate timeIntervalSinceReferenceDate];
    _outfile = [documentsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"asrAudio-%f.pcm",timestamp]];
    NSLog(@"Output file:%@", _outfile);
    
    // 如果文件不存在
    if (![[NSFileManager defaultManager] fileExistsAtPath:_outfile])
    {
        [[NSFileManager defaultManager] createFileAtPath:_outfile contents:nil attributes:nil];
    }
}

- (void) outToFile:(NSData *)data
{
    if (data) {
        NSFileHandle *fileHandle = [NSFileHandle fileHandleForUpdatingAtPath:_outfile];
        [fileHandle seekToEndOfFile];
        [fileHandle writeData:data];
        [fileHandle closeFile];
    }
}
#endif

// Dictionary to JSON
+ (NSString *)convertToJsonData:(NSDictionary *)dict {
    
    NSError *error;
    NSData *jsonData = [NSJSONSerialization dataWithJSONObject:dict options:NSJSONWritingPrettyPrinted error:&error];
    
    NSString *jsonString;
    if (!jsonData) {
        NSLog(@"%@",error);
    }else{
        jsonString = [[NSString alloc]initWithData:jsonData encoding:NSUTF8StringEncoding];
    }
    
    NSMutableString *mutStr = [NSMutableString stringWithString:jsonString];
    NSRange range = {0,jsonString.length};
    
    //去掉字符串中的空格
    [mutStr replaceOccurrencesOfString:@" " withString:@"" options:NSLiteralSearch range:range];
    NSRange range2 = {0,mutStr.length};
    
    //去掉字符串中的换行符
    [mutStr replaceOccurrencesOfString:@"\n" withString:@"" options:NSLiteralSearch range:range2];
    
    return mutStr;
}

// JSON to dictionary
+ (NSDictionary *)dictionaryWithJsonString:(NSString *)jsonString {
    if (jsonString == nil) {
        return nil;
    }
    
    NSData *jsonData = [jsonString dataUsingEncoding:NSUTF8StringEncoding];
    NSError *err;
    NSDictionary *dic = [NSJSONSerialization JSONObjectWithData:jsonData
                                                        options:NSJSONReadingMutableContainers
                                                          error:&err];
    if(err)
    {
        NSLog(@"json解析失败：%@",err);
        return nil;
    }
    return dic;
}

@end
