//
//  JDAISASRDefines.h
//  JDAISpeechManager
//
//  Created by 满达 on 2018/6/27.
//  Copyright © 2018年 Beijing Jingdong Shangke Information Technology Co., Ltd. All rights reserved.
//

// SDK Version: 2.1.100

#ifndef JDAISASRDefines_h
#define JDAISASRDefines_h

#import <Foundation/Foundation.h>

#pragma mark - ASR Command
// 语音识别命令
extern NSString *const JDAIS_ASR_CMD_START;                 // 开始识别
extern NSString *const JDAIS_ASR_CMD_CANCEL;                // 取消识别
extern NSString *const JDAIS_ASR_CMD_STOP;                  // 停止识别

#pragma mark - ASR Delegate
// JDAISpeechManager ASR状态回调
@protocol JDAISClientASRDelegate<NSObject>
- (void)onEventSpeechRecognition:(NSString *)event withResult:(NSString *)result withData:(NSData *)data;
@end

#pragma mark - Event of the Delegate above
// 回调事件
extern NSString *const EV_SJDAIS_ASR_SpeechStart;           // 准备就绪，开始录音
extern NSString *const EV_SJDAIS_ASR_SpeechBegin;           // 用户开始说话
extern NSString *const EV_SJDAIS_ASR_SpeechEnd;             // 用户结束说话
extern NSString *const EV_SJDAIS_ASR_SpeechVolume;          // 音量回调
extern NSString *const EV_SJDAIS_ASR_SpeechParitalResult;   // 临时识别结果
extern NSString *const EV_SJDAIS_ASR_SpeechResult;          // 最终结果

#pragma mark - 设定采样率
typedef enum TJDAISVoiceRecognizeSampleRateFlags
{
    EVoiceRecognizeSampleRate8K  = 8000,
    EVoiceRecognizeSampleRate16K = 16000,
}TJDAISVoiceRecognizeSampleRateFlags;

#pragma mark - 云端VAD开启关闭
typedef enum TJDAISVoiceRecognizeVADEnabled
{
    EVoiceRecognizeVADEnabledNO = 0,
    EVoiceRecognizeVADEnabledYES,
}TJDAISVoiceRecognizeVADEnabled;

#pragma mark - 本地VAD开启关闭
typedef enum TJDAISVoiceRecognizeLocalVADEnabled
{
    EVoiceRecognizeLocalVADEnabledNO = 0,
    EVoiceRecognizeLocalVADEnabledYES,
}TJDAISVoiceRecognizeLocalVADEnabled;

#pragma mark - 识别类型
typedef enum TJDAISVoiceRecognizeType
{
    EVoiceRecognizeRealTime = 0,                          // 实时识别
    EVoiceRecognizeOneSentence,                           // 一句话识别
}TJDAISVoiceRecognizeType;

#pragma mark - 是否录音
typedef enum TJDAISVoiceRecognizeRecordAudio
{
    EVoiceRecognizeRecordAudioNO = 0,
    EVoiceRecognizeRecordAudioYes,                         
}TJDAISVoiceRecognizeRecordAudio;

#pragma mark - 是否打开后处理
typedef enum TJDAISVoiceRecognizePostProcess
{
    EVoiceRecognizePostProcessNO = -1,
    EVoiceRecognizePostProcessDefault,
    EVoiceRecognizePostProcessYes,
}TJDAISVoiceRecognizePostProcess;

#pragma mark - 是否打开实时部分识别结果
typedef enum TJDAISVoiceRecognizePartialResult
{
    EVoiceRecognizePartialResultNO = -1,
    EVoiceRecognizePartialResultDefault,
    EVoiceRecognizePartialResultYes,
}TJDAISVoiceRecognizePartialResult;

#pragma mark - 是否打开识别结果带标点功能
typedef enum TJDAISVoiceRecognizePunc
{
    EVoiceRecognizePuncNO = -1,
    EVoiceRecognizePuncDefault,
    EVoiceRecognizePuncYes,
}TJDAISVoiceRecognizePunc;

#pragma mark - 是否打开动态语言模型功能
typedef enum TJDAISVoiceRecognizeDynamicLm
{
    EVoiceRecognizeDynamicLmNO = -1,
    EVoiceRecognizeDynamicLmDefault,
    EVoiceRecognizeDynamicLmYes,
}TJDAISVoiceRecognizeDynamicLm;

extern const float            kJDAIS_ASR_VAD_START_DELAY_DEF_VAL; // 0.5f
extern const float            kJDAIS_ASR_VAD_END_DELAY_DEF_VAL;   // 5.0f

#pragma mark - 语音识别错误通知状态
extern const NSInteger  EVRClientErrorCodeOK;                      //= 0;                 // 识别成功
extern const NSInteger  EVRClientErrorCodeAudioRecorderException;  //= -1001;             // 录音异常，录音机被占用或者录音设备异常
extern const NSInteger  EVRClientErrorCodePermisionException;      //= -1002;             // 权限异常，未声明录音权限或者权限被其他软件限制
extern const NSInteger  EVRClientErrorCodeNetworkConnectionFailed; //= -1003;             // 网络连接失败，没有网络权限或者网络未连接
extern const NSInteger  EVRClientErrorCodeNetworkTimeout;          //= -1004;             // 网络超时，网络质量差
extern const NSInteger  EVRClientErrorCodeAudioTooLong;            //= -1005;             // 语音太长，说话事件超出限制
extern const NSInteger  EVRClientErrorCodeNoRecognize;             //= -1006;             // 没有识别到语音，用户没有说话
extern const NSInteger  EVRClientErrorCodeServerError;             //= -1007;             // 服务器端错误，服务出错
extern const NSInteger  EVRClientErrorCodeClientError;             //= -1008;             // 客户端错误，参数配置出错，集成过程中传参数设置
extern const NSInteger  EVRClientErrorCodeAudioFormatError;        //= -1009;             // speech format error.
extern const NSInteger  EVRClientErrorCodeAuthError;               //= -1010;             // 认证失败, token验证等非法
extern const NSInteger  EVRClientErrorCodeAudioHeaderError;        //= -1011;             // speech data file header error.
extern const NSInteger  EVRClientErrorCodeSampleRateChannelError;  //= -1012;             // speech sample rate or channel error.
extern const NSInteger  EVRClientErrorCodeVADFaileError;           //= -1013;             // VAD初始化失败
extern const NSInteger  EVRClientErrorCodeJDCloudFaileError;       //= -1014;             // 京东云网关系统错误

#endif /* JDAISASRDefines_h */
